//
//Code block for FileActSnapinElement.cs
// 
using System;
using System.IO;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Configuration;
using System.Runtime.InteropServices;
using MMCTest;
using Ironring.Management.MMC;


namespace MMCTest 
{
	/// <summary>
	/// This is the class which uses the DirecotryNode created above to populate it on to the MMC.
	/// </summary>
	public class FileActSnapinElement 
	{
		public FileActSnapinElement() 
		{
		}
		public void CreateDirectoryNode(SnapinBase snapinBase, string parentDirectory, BaseNode parentNode) 
		{
			DirectoryInfo directoryInfo=new DirectoryInfo(parentDirectory);

			//Create DirectoryNode with all the default properties set
			DirectoryNode directoryNode=new DirectoryNode(snapinBase);
			directoryNode.DisplayName=directoryInfo.Name;
			directoryNode.Properties.Add("NodePath", directoryInfo.Parent.FullName + "\\" + directoryInfo.Name);
			directoryNode.Properties.Add("NodeResultName","");
			directoryNode.Properties.Add("EventTarget","");
			directoryNode.CreateDataTableStructure();
			directoryNode.FillDataTableValues();
			parentNode.AddChild(directoryNode);

			//Add Menu and EventHandler for Menu for DirectoryNode
			MenuItem topMenuItemFileActProperties=new MenuItem("FileAct Properties", "FileAct Properties for selected the item", new MenuCommandHandler(OnMenuFileActPropertiesClick));
			directoryNode.AddTopMenuItem(topMenuItemFileActProperties);
			MenuItem topMenuItemRefresh=new MenuItem("Refresh", "Refresh the contents",  new MenuCommandHandler(OnMenuRefreshClick));
			directoryNode.AddTopMenuItem(topMenuItemRefresh);

			//Add other Event Handlers for DirectoryNode
			directoryNode.OnSelectScopeEvent+=new NodeNotificationHandler(OnSelectScopeEvent);
			directoryNode.OnSelectResultEvent+=new NodeNotificationHandler(OnSelectResultEvent);

			foreach(DirectoryInfo subDirectoryInfo in directoryInfo.GetDirectories()) 
			{
				CreateDirectoryNode(snapinBase, subDirectoryInfo.Parent.FullName + "\\" + subDirectoryInfo.Name, directoryNode);
			}
		}

		//**********************************************************************//
		//Event Handlers
		//Handles events generated by nodes and the result pane
		//
		//**********************************************************************//

		//**********************************************************************//
		//MMC Event Handlers
		//**********************************************************************//
		/// <summary>
		/// This method is called when the node element is selected.
		/// </summary>
		protected void OnSelectScopeEvent(object sender, NodeEventArgs args) 
		{
			DirectoryNode directoryNode=(DirectoryNode) sender;
			directoryNode.Properties["EventTarget"]="Node";
		}

		/// <summary>
		/// This method is called when the node's list/result item is selected.
		/// </summary>
		protected void OnSelectResultEvent(object sender, NodeEventArgs args) 
		{
			DirectoryNode directoryNode=(DirectoryNode) sender;
			directoryNode.Properties["EventTarget"]="NodeResult";
		}

		//**********************************************************************//
		//Menu Event Handlers
		//**********************************************************************//
		/// <summary>
		/// This method is called when menu item "FileAct Properties" is clicked.
		/// </summary>
		public void OnMenuFileActPropertiesClick(object sender, BaseNode arg) 
		{
			DirectoryNode directoryNode=(DirectoryNode)arg;
			uint pItemID;
			IConsole2 console=directoryNode.Snapin.ResultViewConsole;
			IResultData resultData=console as IResultData;
			RESULTDATAITEM resultDataItem=new RESULTDATAITEM();

			try	
			{
				if(directoryNode.Properties["EventTarget"].ToString().Equals("Node")) 
				{
					FormDirectoryProperty formDirectoryProperty=new FormDirectoryProperty();
					formDirectoryProperty.NodeProperties=directoryNode.Properties;
					formDirectoryProperty.ShowDialog();
				}
				else 
				{
					for(int loopCounter=0; loopCounter<directoryNode.Table.Rows.Count; loopCounter++) 
					{
						resultData.FindItemByLParam(directoryNode.Cookie | (loopCounter+1 << 16), out pItemID);
						resultDataItem.nCol=0;
						resultDataItem.mask=(uint)RDI.STATE | (uint)RDI.PARAM;
						resultDataItem.itemID=(int)pItemID;
						resultData.GetItem(ref resultDataItem);
						if(resultDataItem.nState==3) 
						{
							directoryNode.Properties["NodeResultName"]=directoryNode.Table.Rows[loopCounter][0];
							break;
						}
					}

					FormFileProperty formFileProperty=new FormFileProperty();
					formFileProperty.NodeProperties=directoryNode.Properties;
					formFileProperty.ShowDialog();
				}
			}
			catch(Exception ex)	
			{
				throw ex;
			}
		}

		/// <summary>
		/// This method is called when menu item "Refresh" is clicked.
		/// </summary>
		public void OnMenuRefreshClick(object sender, BaseNode arg) 
		{
			string selectedFileName=null;
			DirectoryNode directoryNode=(DirectoryNode)arg;
			uint pItemID;
			IConsole2 console=directoryNode.Snapin.ResultViewConsole;
			IResultData resultData=console as IResultData;
			RESULTDATAITEM resultDataItem=new RESULTDATAITEM();

			try	
			{
				if(!directoryNode.Properties["EventTarget"].ToString().Equals("Node")) 
				{
					for(int loopCounter=0; loopCounter<directoryNode.Table.Rows.Count; loopCounter++) 
					{
						resultData.FindItemByLParam(directoryNode.Cookie | (loopCounter+1 << 16), out pItemID);
						resultDataItem.nCol=0;
						resultDataItem.mask=(uint)RDI.STATE | (uint)RDI.PARAM;
						resultDataItem.itemID=(int)pItemID;
						resultData.GetItem(ref resultDataItem);
						if(resultDataItem.nState==3) 
						{
							selectedFileName=directoryNode.Table.Rows[loopCounter][0].ToString();
							break;
						}
					}
				}

				resultData.DeleteAllRsltItems();
				directoryNode.Table.Rows.Clear();
				directoryNode.Table.AcceptChanges();
				directoryNode.FillDataTableValues();
				directoryNode.OnShowData(resultData);

				if(!directoryNode.Properties["EventTarget"].ToString().Equals("Node")) 
				{
					for(int loopCounter=0; loopCounter<directoryNode.Table.Rows.Count; loopCounter++) 
					{
						if(directoryNode.Table.Rows[loopCounter][0].ToString().Equals(selectedFileName))
						{
							resultData.FindItemByLParam(directoryNode.Cookie | (loopCounter+1 << 16), out pItemID);
							resultDataItem.mask=(uint)RDI.STATE;
							resultDataItem.itemID=(int)pItemID;
							resultDataItem.nState=3;
							resultData.SetItem(ref resultDataItem);
							break;
						}
					}
				}
			}
			catch(Exception ex)	
			{
				throw ex;
			}
		}

	}
}
