using System;
using System.Runtime.InteropServices;
using Ironring.Management.MMC;
using System.Drawing;
using System.Data;


namespace MMCTest
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	/// 
	
	[
		SnapinIn("MMC Library Test Snapin", "Ironring Software", "2.1.0.100"),
		ProgId("MMCLibrary.TestSnapin"),
		Guid("B7F2CD77-FA26-4245-A086-984C74AF7884")
	]
	public class TestSnapin : SnapinBase
	{
        protected ImageList m_SharedResultViewImages;

		public TestSnapin()
		{
            try
            {
                // add images to the global snapin scope image collection

                AddImage("MMCTest.images.Closed.ico");
                AddImage("MMCTest.images.Open.ico");
                AddImage("MMCTest.images.Happy.ico");

                // populate the node result view collection
                m_SharedResultViewImages = new ImageList();
                m_SharedResultViewImages.Add("MMCTest.images.Open.ico");
                m_SharedResultViewImages.Add("MMCTest.images.Happy.ico");

                // ste the default result view image control 
                BaseNode.DefaultResultPaneImages = m_SharedResultViewImages;

                // create my base node
                BaseNode rootnode = new BaseNode(this);
                rootnode.DisplayName = "My Test Snapin";
                base.RootNode = rootnode;

                // create the sub nodes
                CreateHTMLNode(rootnode);
                CreateOCXNode();
                CreateForm1Node();
                CreateReportNode();
                CreateNodeTree();
                CreateForm2Node();
				FileActSnapinElement fileActSnapinElement=new FileActSnapinElement();
				fileActSnapinElement.CreateDirectoryNode(this,@"C:\FileActDemo",rootnode);

            }
            catch(Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Initialization exception - " + e.ToString());
            }
        }


		protected void CreateNodeTree()
		{
			BaseNode node = new BaseNode(this);
			node.DisplayName = "The BaseNode forest";
			base.RootNode.AddChild(node);

            BaseNode childnode = null;

			childnode = new BaseNode(this);
			childnode.DisplayName = "1";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "2";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "3";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "4";
            node.AddChild(childnode);

            MenuItem itemTask = new MenuItem("A Task", "Do a task", new MenuCommandHandler(MenuHandler));
            MenuItem itemView = new MenuItem("A View Command", "Do something to the view", new MenuCommandHandler(MenuHandler));

			childnode = new BaseNode(this);
			childnode.DisplayName = "5";
            childnode.AddTaskMenuItem(itemTask);
            childnode.AddViewMenuItem(itemView);
            node.AddChild(childnode);
            node = childnode;

			childnode = new BaseNode(this);
			childnode.DisplayName = "6";
            childnode.AddTaskMenuItem(itemTask);
            childnode.AddViewMenuItem(itemView);
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "7";
            node.AddChild(childnode);
			node = childnode;

			childnode = new BaseNode(this);
			childnode.DisplayName = "8";
            node.AddChild(childnode);

			childnode = new BaseNode(this);
			childnode.DisplayName = "9";
            node.AddChild(childnode);
		}

        protected HTMLNode CreateHTMLNode(BaseNode parent)
		{
			HTMLNode node = new HTMLNode(this);
			node.DisplayName = "Foo - HTML Node";
			node.Url = "http://www.ironringsoftware.com";
            
            parent.AddChild(node);

			// context menu support

			MenuItem item = new MenuItem("Top level menu Item", "Something important because its on top!", new MenuCommandHandler(MenuHandler));
			node.AddTopMenuItem(item);
			item.Handler += new MenuCommandHandler(MenuHandler2); // add a second handler for the event

			item = new MenuItem("New Something", "add a new something", new MenuCommandHandler(NewNodeMenuHandler));
			node.AddNewMenuItem(item);

			item = new MenuItem("A Task", "Do a task", new MenuCommandHandler(MenuHandler));
			node.AddTaskMenuItem(item);

			item = new MenuItem("A View Item", "Something view related", new MenuCommandHandler(MenuHandler));
			node.AddViewMenuItem(item);

            node.AddPropertyPage("Page 1", Type.GetType("MMCTest.UserControl1"));
            node.AddPropertyPage("Page 2", Type.GetType("MMCTest.UserControl2"));
            node.AddPropertyPage("Page 3", Type.GetType("MMCTest.UserControl1"));
            node.AddPropertyPage("Page 4", Type.GetType("MMCTest.UserControl2"));
            node.AddPropertyPage("Page 5", Type.GetType("MMCTest.UserControl1"));

            return node;
        }

        // TBD: change the prototype - BaseNode -> MenuHandlerArgs and have BaseNode be a member of that 
		protected void MenuHandler(object sender, BaseNode arg)
		{
            System.Windows.Forms.MessageBox.Show("You did something!");
		}

		protected void MenuHandler2(object sender, BaseNode arg)
		{
            System.Windows.Forms.MessageBox.Show("You did something else!");
        }
        protected void NewNodeMenuHandler(object sender, BaseNode arg)
        {
            // dynamically insert anew node under neigth the sender
            HTMLNode node = CreateHTMLNode(arg);
            node.Insert(arg);

        }

		protected void CreateOCXNode()
		{
			OCXNode node = new OCXNode(this);
			node.DisplayName = "Bar - OCX Node";
			// setup to use the calendar control
			node.OCXGuid = new Guid("8E27C92B-1264-101C-8A2F-040224009C02");
			base.RootNode.AddChild(node);
		}


        protected void CreateForm1Node()
        {
            FormNode node = new FormNode(this);
            node.DisplayName = "Well Formed! - Form Node";
            node.ControlType = Type.GetType("MMCTest.UserControl1");
            base.RootNode.AddChild(node);
        }

        protected void CreateForm2Node()
        {
            FormNode node = new FormNode(this);
            node.DisplayName = "Well Formed again! - Form Node";
            node.ControlType = Type.GetType("MMCTest.UserControl2");
            base.RootNode.AddChild(node);
        }
      
        protected void CreateReportNode()
		{
			DataTable theDataTable = new DataTable();

			// Declare variables for DataColumn and DataRow objects.
			DataColumn myDataColumn;
			DataRow myDataRow;
 
			// Create new DataColumn, set DataType, ColumnName and add to DataTable.    
			myDataColumn = new DataColumn();
			myDataColumn.DataType = System.Type.GetType("System.Int32");
			myDataColumn.ColumnName = "item";
			myDataColumn.ReadOnly = true;
			myDataColumn.Unique = true;
			// Add the Column to the DataColumnCollection.
			theDataTable.Columns.Add(myDataColumn);
 
			// Create second column.
			myDataColumn = new DataColumn();
			myDataColumn.DataType = System.Type.GetType("System.String");
			myDataColumn.ColumnName = "Name";
			myDataColumn.AutoIncrement = false;
			myDataColumn.Caption = "Name";
			myDataColumn.ReadOnly = false;
			myDataColumn.Unique = false;
			// Add the column to the table.
			theDataTable.Columns.Add(myDataColumn);

            // Create third column.
            myDataColumn = new DataColumn();
            myDataColumn.DataType = System.Type.GetType("System.String");
            myDataColumn.ColumnName = "Description";
            myDataColumn.AutoIncrement = false;
            myDataColumn.Caption = "Description";
            myDataColumn.ReadOnly = false;
            myDataColumn.Unique = false;
            // Add the column to the table.
            theDataTable.Columns.Add(myDataColumn);


			// Make the ID column the primary key column.
			DataColumn[] PrimaryKeyColumns = new DataColumn[1];
			PrimaryKeyColumns[0] = theDataTable.Columns["item"];
			theDataTable.PrimaryKey = PrimaryKeyColumns;
 
 
			// Create three new DataRow objects and add them to the DataTable
			for (int i = 0; i <= 200; i++)
			{
				myDataRow = theDataTable.NewRow();
				myDataRow["item"] = i;
				myDataRow["Name"] = "I Am item " + i;
                myDataRow["Description"] = "I Am a desription for item " + i;
                theDataTable.Rows.Add(myDataRow);
			}

			// setup the node
			ReportNode node = new ReportNode(this);
			node.DisplayName = "Baz - The Report Node";
			node.Table = theDataTable;
            
            base.RootNode.AddChild(node);
		}
	}
}

